﻿using System;
using System.Text;

using System.IO;

using Popolo.ThermalLoad;
using Popolo.Weather;

namespace SampleModels
{
  class Program
  {
    /// <summary>省エネルギー法　例題戸建住宅の計算</summary>
    /// <param name="args"></param>
    static void Main(string[] args)
    {
      //1年間の東京の気象データを作成
      double[] dbt, hmd, rad;
      bool[] isFair;
      RandomWeather.MakeWeather(1, RandomWeather.Location.Tokyo, 1, out dbt, out hmd, out rad, out isFair);

      //建物を作成
      BuildingThermalModel house = makeHouse();
      house.InitializeAirState(20, 0.010);

      //外部出力ファイル
      using (StreamWriter sWriter = new StreamWriter("output.csv", false, Encoding.GetEncoding("Shift_JIS")))
      {
        //タイトル行
        sWriter.Write("日時,外気乾球温度[C],外気絶対湿度[kg/kg]");
        for (int i = 0; i < 5; i++)
          sWriter.Write("," +
            house.MultiRoom[0].Zones[i].Name + "乾球温度[C]," +
            house.MultiRoom[0].Zones[i].Name + "絶対湿度[kg/kg]," +
            house.MultiRoom[0].Zones[i].Name + "顕熱負荷[W]," +
            house.MultiRoom[0].Zones[i].Name + "潜熱負荷[W]");
        sWriter.WriteLine();

        //年間の計算を実行
        Sun sun = new Sun(Sun.City.Tokyo);
        DateTime now = new DateTime(1999, 1, 1, 0, 0, 0);
        for (int hour = 0; hour < 8760; hour++)
        {
          //計算進捗をコンソールに書き出し
          if (now.Hour == 0) Console.WriteLine(now.ToShortDateString());

          //気象条件を設定
          sun.Update(now);
          sun.SeparateGlobalHorizontalRadiation(rad[hour], Sun.SeparationMethod.Erbs);  //直参分離
          house.UpdateOutdoorCondition(now, sun, dbt[hour], 0.001 * hmd[hour], 0);  //夜間放射は無視

          //室温制御設定
          //基本、温湿度は成り行き
          for (int i = 0; i < 5; i++)
          {
            house.ControlHeatSupply(0, i, 0);   //加熱冷却無し
            house.ControlWaterSupply(0, i, 0);  //除加湿無し
          }
          //（夏と冬の夜間の就寝前は寝室の温度を制御する）
          if ((now.Month == 7 || now.Month == 8) && 18 < now.Hour)
          {
            house.ControlDrybulbTemperature(0, 2, 26);
            house.ControlDrybulbTemperature(0, 3, 26);
            house.ControlDrybulbTemperature(0, 4, 26);
          }
          else if ((now.Month == 1 || now.Month == 2) && 18 < now.Hour)
          {
            house.ControlDrybulbTemperature(0, 2, 22);
            house.ControlDrybulbTemperature(0, 3, 22);
            house.ControlDrybulbTemperature(0, 4, 22);
          }

          //標準では0.5回/hの換気
          for (int i = 0; i < 5; i++)
            house.SetVentilationRate(0, i, house.MultiRoom[0].Zones[i].AirMass / 1.2 * 0.5);

          //温湿度更新
          house.ForecastHeatTransfer();   //顕熱更新
          house.ForecastWaterTransfer();  //潜熱更新

          //28度を超えたら窓開け換気（換気回数5回/h相当とみなす）
          bool hasWindowOpened = false;
          for (int i = 0; i < 5; i++)
          {
            if (28 < house.MultiRoom[0].Zones[i].Temperature)
            {
              hasWindowOpened = true;
              house.SetVentilationRate(0, i, house.MultiRoom[0].Zones[i].AirMass / 1.2 * 5);
            }
          }
          //窓が開いた場合には温度を再計算
          if (hasWindowOpened) house.ForecastHeatTransfer();

          //温湿度を確定
          house.FixState(); //確定

          //1時間進める
          now = now.AddHours(1);

          //計算結果書き出し
          sWriter.Write(now.ToString() + "," + dbt[hour] + "," + (0.001 * hmd[hour]));
          for (int i = 0; i < 5; i++)
            sWriter.Write("," +
              house.MultiRoom[0].Zones[i].Temperature + "," +
              house.MultiRoom[0].Zones[i].HumidityRatio + "," +
              house.MultiRoom[0].Zones[i].HeatSupply + "," +
              house.MultiRoom[0].Zones[i].WaterSupply * 2500000);
          sWriter.WriteLine();
        }
      }
    }

    /// <summary>戸建住宅モデルを作成する</summary>
    /// <returns>戸建住宅モデル</returns>
    private static BuildingThermalModel makeHouse()
    {
      //傾斜面を作成（北東西南垂直面+水平面）
      Incline incN = new Incline(Incline.Orientation.N, 0.5 * Math.PI);
      Incline incE = new Incline(Incline.Orientation.E, 0.5 * Math.PI);
      Incline incW = new Incline(Incline.Orientation.W, 0.5 * Math.PI);
      Incline incS = new Incline(Incline.Orientation.S, 0.5 * Math.PI);
      Incline incH = new Incline(Incline.Orientation.N, 0);

      //壁構成を作成
      WallLayer[] exWL = new WallLayer[5];  //外壁
      exWL[0] = new WallLayer(WallLayer.Materials.Mortar, 0.030);   //モルタル
      exWL[1] = new WallLayer(WallLayer.Materials.Plywood, 0.009);  //合板
      exWL[2] = new WallLayer(WallLayer.Materials.AirGap, 0.150);   //非密閉空気層
      exWL[3] = new WallLayer(WallLayer.Materials.GlassWoolInsulation_24K, 0.050);  //グラスウール25K
      exWL[4] = new WallLayer(WallLayer.Materials.PlasterBoard, 0.012); //石膏ボード
      WallLayer[] flWL1 = new WallLayer[2];  //1F床
      flWL1[0] = new WallLayer(WallLayer.Materials.Plywood, 0.022);  //合板
      flWL1[1] = new WallLayer(WallLayer.Materials.GlassWoolInsulation_24K, 0.050);  //グラスウール25K
      WallLayer[] flWL2 = new WallLayer[4];  //2F床
      flWL2[0] = new WallLayer(WallLayer.Materials.Carpet, 0.015);  //カーペット
      flWL2[1] = new WallLayer(WallLayer.Materials.Plywood, 0.012);  //合板
      flWL2[2] = new WallLayer(WallLayer.Materials.AirGap, 0.150);   //非密閉空気層
      flWL2[3] = new WallLayer(WallLayer.Materials.PlasterBoard, 0.012); //石膏ボード
      WallLayer[] rfWL = new WallLayer[4];  //屋根
      rfWL[0] = new WallLayer(WallLayer.Materials.Plywood, 0.012);  //合板
      rfWL[1] = new WallLayer(WallLayer.Materials.AirGap, 0.150);   //非密閉空気層
      rfWL[2] = new WallLayer(WallLayer.Materials.Plywood, 0.012);  //合板
      rfWL[3] = new WallLayer(WallLayer.Materials.Kawara, 0.012);  //瓦
      WallLayer[] inWL = new WallLayer[3];  //内壁
      inWL[0] = new WallLayer(WallLayer.Materials.PlasterBoard, 0.012); //石膏ボード
      inWL[1] = new WallLayer(WallLayer.Materials.AirGap, 0.150);   //非密閉空気層
      inWL[2] = new WallLayer(WallLayer.Materials.PlasterBoard, 0.012); //石膏ボード

      //窓を作成:すべて透明フロート複層
      Window[] windows = new Window[8];
      windows[0] = new Window(4.59, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incS);  //和室南
      windows[1] = new Window(3.47*2, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incS);  //LD南
      windows[2] = new Window(2.15, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incE);  //LD東
      windows[3] = new Window(0.99, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incW);  //寝室西
      windows[4] = new Window(1.73, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incS); //寝室南
      windows[5] = new Window(3.22, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incS);  //子供部屋1南
      windows[6] = new Window(3.22, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incS);  //子供部屋2南
      windows[7] = new Window(0.66, new double[] { 0.7, 0.7 }, new double[] { 0.04, 0.04 }, incE);  //子供部屋2東

      //壁を作成
      Wall[] walls = new Wall[25]; //外壁14枚+間仕切り11枚
      walls[0] = new Wall(3.64 * 2.9, exWL);  //和室西
      walls[1] = new Wall(4.55 * 2.9 - windows[0].Area, exWL);  //和室南
      walls[2] = new Wall(5.915 * 2.9 - windows[1].Area, exWL); //LD南
      walls[3] = new Wall(3.64 * 2.9 - windows[2].Area, exWL);  //LD東
      walls[4] = new Wall(3.64 * 2.7 - windows[3].Area, exWL);  //寝室西
      walls[5] = new Wall(3.64 * 2.7 - windows[4].Area, exWL);  //寝室南
      walls[6] = new Wall(2.9575 * 2.7 - windows[5].Area, exWL);//子供部屋1南
      walls[7] = new Wall(2.9575 * 2.7 - windows[6].Area, exWL);//子供部屋2南
      walls[8] = new Wall(3.64 * 2.7 - windows[7].Area, exWL);  //子供部屋2東
      walls[9] = new Wall(4.55 * 3.64, flWL1);  //和室床
      walls[10] = new Wall(5.915 * 3.64, flWL1);  //LD床
      walls[11] = new Wall(3.64 * 3.64, rfWL);  //寝室屋根
      walls[12] = new Wall(2.9575 * 3.64, rfWL);  //子供部屋1屋根
      walls[13] = new Wall(2.9575 * 3.64, rfWL);  //子供部屋2屋根
      walls[14] = new Wall(3.64 * 2.9, inWL);  //和室-LD 間
      walls[15] = new Wall(3.64 * 3.64, flWL2);  //和室-寝室 間
      walls[16] = new Wall(2.9575 * 3.64, flWL2);  //LD-子供部屋1 間
      walls[17] = new Wall(2.9575 * 3.64, flWL2);  //LD-子供部屋2 間
      walls[18] = new Wall(3.64 * 2.7, inWL);  //寝室-子供部屋1 間
      walls[19] = new Wall(3.64 * 2.7, inWL);  //子供部屋1-子供部屋2 間
      walls[20] = new Wall(4.55 * 2.9, inWL);  //和室-廊下 間
      walls[21] = new Wall(5.915 * 2.9, inWL);  //LD-廊下 間
      walls[22] = new Wall(3.64 * 2.7, inWL);  //寝室-廊下 間
      walls[23] = new Wall(2.9575 * 2.7, inWL);  //子供部屋1-廊下 間
      walls[24] = new Wall(2.9575 * 2.7, inWL);  //子供部屋2-廊下 間
      
      for (int i = 0; i < walls.Length; i++)
      {
        walls[i].LongWaveEmissivityF = walls[i].LongWaveEmissivityB = 0.9;
        walls[i].ShortWaveAbsorptanceF = walls[i].ShortWaveAbsorptanceB = 0.8;
      }

      //ゾーンを作成//廊下や便所などは計算対象外とする
      Zone[] zones = new Zone[5];
      zones[0] = new Zone("和室", 4.55 * 3.64 * 2.9 * 1.2);
      zones[1] = new Zone("LD", 5.915 * 3.64 * 2.9 * 1.2);
      zones[2] = new Zone("寝室", 3.64 * 3.64 * 2.7 * 1.2);
      zones[3] = new Zone("子供部屋1", 2.9575 * 3.64 * 2.7 * 1.2);
      zones[4] = new Zone("子供部屋2", 2.9575 * 3.64 * 2.7 * 1.2);
      //0.5回/hの換気
      for (int i = 0; i < zones.Length; i++)
        zones[i].VentilationRate = zones[i].AirMass / 3600d * 0.2;

      //MultiRoomsを作成
      MultiRooms mRoom = new MultiRooms(1, zones, walls, windows);

      //ゾーン設定
      for (int i = 0; i < zones.Length; i++)
        mRoom.AddZone(0, i);
      //壁設定
      //外壁（垂直面）
      mRoom.AddWall(0, 0, true); mRoom.SetOutsideWall(0, false, incW);
      mRoom.AddWall(0, 1, true); mRoom.SetOutsideWall(1, false, incS);
      mRoom.AddWall(1, 2, true); mRoom.SetOutsideWall(2, false, incS);
      mRoom.AddWall(1, 3, true); mRoom.SetOutsideWall(3, false, incE);
      mRoom.AddWall(2, 4, true); mRoom.SetOutsideWall(4, false, incW);
      mRoom.AddWall(2, 5, true); mRoom.SetOutsideWall(5, false, incS);
      mRoom.AddWall(3, 6, true); mRoom.SetOutsideWall(6, false, incS);
      mRoom.AddWall(4, 7, true); mRoom.SetOutsideWall(7, false, incS);
      mRoom.AddWall(4, 8, true); mRoom.SetOutsideWall(8, false, incE);
      //外壁（床下・屋根）
      mRoom.AddWall(0, 9, true); mRoom.UseAdjacentSpaceFactor(9, false, 0.7); //床下は隣室温度差係数で表現
      mRoom.AddWall(1, 10, true); mRoom.UseAdjacentSpaceFactor(10, false, 0.7); //床下は隣室温度差係数で表現
      mRoom.AddWall(2, 11, true); mRoom.SetOutsideWall(11, false, incH);
      mRoom.AddWall(3, 12, true); mRoom.SetOutsideWall(12, false, incH);
      mRoom.AddWall(4, 13, true); mRoom.SetOutsideWall(13, false, incH);
      //内壁(共用壁)
      mRoom.AddWall(0, 1, 14);  //和室-LD 間
      mRoom.AddWall(0, 2, 15);  //和室-寝室 間
      mRoom.AddWall(1, 3, 16);  //LD-子供部屋1 間
      mRoom.AddWall(1, 4, 17);  //LD-子供部屋2 間
      mRoom.AddWall(2, 3, 18);  //寝室-子供部屋1 間
      mRoom.AddWall(3, 4, 19);  //子供部屋1-子供部屋2 間
      //内壁(廊下)
      mRoom.AddWall(0, 20, true); mRoom.UseAdjacentSpaceFactor(20, false, 0.5); //廊下は隣室温度差係数で表現
      mRoom.AddWall(1, 21, true); mRoom.UseAdjacentSpaceFactor(21, false, 0.5); //廊下は隣室温度差係数で表現
      mRoom.AddWall(2, 22, true); mRoom.UseAdjacentSpaceFactor(22, false, 0.5); //廊下は隣室温度差係数で表現
      mRoom.AddWall(3, 23, true); mRoom.UseAdjacentSpaceFactor(23, false, 0.5); //廊下は隣室温度差係数で表現
      mRoom.AddWall(4, 24, true); mRoom.UseAdjacentSpaceFactor(24, false, 0.5); //廊下は隣室温度差係数で表現
      //窓設定
      mRoom.AddWindow(0, 0);
      mRoom.AddWindow(1, 1);
      mRoom.AddWindow(1, 2);
      mRoom.AddWindow(2, 3);
      mRoom.AddWindow(2, 4);
      mRoom.AddWindow(3, 5);
      mRoom.AddWindow(4, 6);
      mRoom.AddWindow(4, 7);
      //アルベド
      mRoom.Albedo = 0.2;

      return new BuildingThermalModel(new MultiRooms[] { mRoom });
    }

  }
}
